# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from LaTeXPlugin.Logger import debug, error


class Node(object):
	def __init__(self):
		self.__children = []
	
	@property
	def children(self):
		return self.__children
	

class Placeholder(object):
	def __init__(self):
		self.__children = []
	
	@property
	def children(self):
		return self.__children
	
	def set_children(self, children):
		self.__children = children
		

class Command(Node):
	def __init__(self, name, package):
		Node.__init__(self)
		self.name = name
		self.package = package
	
	@property
	def variants(self):
		return self.children
	
#	def _get_variant_by_signature(self, signature):
#		"""
#		Find a variant of this command by the order of its signature (argument type order)
#		
#		Optional argument may be skipped and marked unused (fuzzy matching).
#		
#		_Be aware_: the 'signature' may not be complete. We may get '\cmd[]{' where 'cmd'
#		may be defined like:
#		
#		cmd
#			0
#				Opt
#				Mand
#				Mand
#			1
#				Opt
#				Mand
#				Opt
#				
#		_Problem_: most of the commands have only one variant but we should always run this method
#		because optional arguments must be marked used or unused. This should be done in a more
#		transparent way.
#		"""
#		
#		# FIXME
#		####
#		#return self.variants[0]
#		####
#		
#		debug(self, "get_variant_by_signature", str(signature))
#		
#		#
#		# TODO: this is too complicated...
#		#
#		M_NOT, M_EXACT, M_FUZZY = 0, 1, 2
#		
#		# init
#		matches = []
#		for variant in self.variants:
#			matches.append(M_EXACT)
#		
#		
#		i = 0
#		
#		for variant in self.variants:
#			
#			for j in range(len(signature)):
#				if type(variant.arguments[j]) is not signature[j]:
#					if type(variant.arguments[j]) is OptionalArgument:
#						variant.arguments[j].used = False
#						matches[i] = M_FUZZY
#					else:
#						matches[i] = M_NOT
#				else:
#					if type(variant.arguments[j]) is OptionalArgument:
#						variant.arguments[j].used = True
#					
#			i += 1
#		
#		
#		# FIXME
#		# we need to return here, because otherwise the 'used' isn't set
#		####
#		return self.variants[0]
#		####
#		
#		
#		# exact matches first
#		for i in range(len(matches)):
#			if matches[i] == M_EXACT:
#				return self.variants[i]
#		
#		for i in range(len(matches)):
#			if matches[i] == M_FUZZY:
#				return self.variants[i]
#		
#		raise KeyError
	
	def get_variant_by_signature(self, signature):
		
		for variant in self.variants:
			
			arguments = variant.arguments
			
#			t = []
#			for a in arguments:
#				t.append(type(a))
#			debug(self, "get_variant_by_signature", "variant arguments: " + str(t))
			
			match = True
			
			for i in range(len(signature)):
				if signature[i] is not type(arguments[i]):		# arguments don't match
					if type(arguments[i]) is OptionalArgument:
						# an optional argument may be skipped and marked unused
#						debug(self, "get_variant_by_signature", "argument #%s is optional, marking unused" % i)
						arguments[i].used = False
					else:
						# all other types may not be skipped
#						debug(self, "get_variant_by_signature", "argument #%s is mandatory, no match" % i)
						match = False
				else:											# arguments match
					if type(arguments[i]) is OptionalArgument:
#						debug(self, "get_variant_by_signature", "argument #%s is optional, marking used" % i)
						arguments[i].used = True
			
			# return the first matching variant
			# TODO: does this make sense?
			if match:
				return variant
		
		# no matching variant found
		debug(self, "get_variant_by_signature", "signature: " + str(signature))
		raise KeyError
					


class Variant(Node):
	def __init__(self, command):
		Node.__init__(self)
		self.__command = command
	
	@property
	def command(self):
		return self.__command
	
	@property
	def arguments(self):
		return self.children
	
	@property
	def skeleton(self):
		"""
		Return empty brackets for the arguments of this variant
		"""
		s = ""
		for argument in self.arguments:
			if type(argument) is MandatoryArgument:
				s += "{}"
			elif type(argument) is OptionalArgument:
				s += "[]"
			elif type(argument) is VectorArgument:
				s += "()"
		return s
	
	def get_used_argument(self, index):
		"""
		Returns an argument skipping unused optional arguments
		"""
		#
		# create a new list containing only used arguments
		#
		# TODO: this doesn't seem to work correctly and this can be done in an
		# easier way...
		#
		
		l = [a for a in self.arguments if (type(a) is not OptionalArgument) or (type(a) is OptionalArgument and a.used)]
		return l[index]
	
	def __str__(self):
		return self.command.name + self.skeleton


# TODO: mother class "NormalArgument"

class MandatoryArgument(Node):
	def __init__(self, label):
		Node.__init__(self)
		self.__label = label
	
	@property
	def label(self):
		return self.__label
	
	@property
	def choices(self):
		# placeholders are allowed as choices, so evaluate them here
		children = []
		
		for child in self.children:
			if type(child) is Placeholder:
				children.extend(child.children)
			else:
				children.append(child)
		
		return children
	
	def append_choice(self, choice):
		self.children.append(choice)


class OptionalArgument(Node):
	def __init__(self, label):
		Node.__init__(self)
		self.__label = label
		self.used = True
	
	@property
	def label(self):
		return self.__label
	
	@property
	def choices(self):
		# placeholders are allowed as choices, so evaluate them here
		children = []
		
		for child in self.children:
			if type(child) is Placeholder:
				children.extend(child.children)
			else:
				children.append(child)
		
		return children
	
	def append_choice(self, choice):
		self.children.append(choice)


class VectorArgument(Node):
	def __init__(self):
		Node.__init__(self)
	
	@property
	def components(self):
		return self.children


class Choice(object):
	def __init__(self, value, package=None, doc=None):
		self.__value = value
		self.__package = package
		self.__doc = doc
	
	@property
	def doc(self):
		return self.__doc
	
	@property
	def package(self):
		return self.__package
	
	@property
	def value(self):
		return self.__value


class Component(object):
	def __init__(self, label):
		self.__label = label
		
	@property
	def label(self):
		return self.__label


class CommandStore(dict):
	"""
	"""
	def __init__(self):
		dict.__init__(self)
		self.__placeholders = {}	# <str, []>
		
	def find(self, prefix):
		"""
		Find all commands starting with prefix
		"""
		return [command for command in self.itervalues() if command.name.startswith(prefix)]
	
	def register_placeholder(self, key, node):
		try:
			nodes = self.__placeholders[key]
			nodes.append(node)
		except KeyError:
			self.__placeholders[key] = [node]
		
		debug(self, "register_placeholder", "Registered '%s'" % key)
	
	def fill_placeholder(self, key, nodes):
		try:
			for node in self.__placeholders[key]:
				node.set_children(nodes)
				
		except KeyError:
			error(self, "fill_placeholder", "Placeholder '%s' not registered" % key)


from copy import deepcopy
from CommandsParser import CommandsParser
from LaTeXPlugin.Installation import Installation
from LaTeXPlugin.Logger import debug


class CommandStoreFactory(object):
	"""
	This factory produces instances of a CommandStore. It is implemented as a singleton class.
	
	The advantage of this is that we don't have to parse definitions.xml for each new 
	LaTeXDocumentHelper.
	"""
	def __new__(type):
		if not '_instance' in type.__dict__:
			type._instance = object.__new__(type)
		return type._instance
	
	def __init__(self):
		if not '_ready' in dir(self):
			 debug(self, "__init__")
			 
			 self._commandStore = CommandStore()
			 
			 parser = CommandsParser()
			 parser.parse(self._commandStore, "%s/autocomplete/commands.xml" % Installation().path)
			 
			 self._ready = True
	
	def getCommandStore(self):
		return deepcopy(self._commandStore)
	