from LaTeXPlugin.common import ConditionalSource
from LaTeXPlugin.sourcebuilder import *

# TODO: \rotatebox[Options][Angle]{} allows rotating with custom origin

class GraphicsSourceBuilder(object):
	"""
	"""
	def __init__(self):
		self.filename = None
		
		self.center = True
		
		# scale
		self.h_scale = 1.0
		self.v_scale = 1.0
		
		# resize
		self.resize_width = None
		self.resize_height = None
		self.resize_unit = None
		
		# reflect
		self.reflect = False
		
		# rotate
		self.rotate = 0.0
		
		# floating object
		self.float = True
		
		self.placement_h = False
		self.placement_t = True
		self.placement_b = True
		self.placement_p = True
		
		self.caption = None
		self.label = None
		
		self.working_directory = None
		
	def get_source(self):
		# scale, resize, rotate
		options = {}
		if (self.h_scale != 1.0 or self.v_scale != 1.0) and self.h_scale == self.v_scale:
			options["scale"] = round(self.h_scale, 2)
		if self.resize_width:
			options["width"] = "%s%s" % (round(self.resize_width, 2), self.resize_unit)
		if self.resize_height:
			options["height"] = "%s%s" % (round(self.resize_height, 2), self.resize_unit)
		if self.rotate != 0.0:
			options["angle"] = round(self.rotate, 2)
			
		# try relativize filename to work directory
		filename = self.filename
		
		if self.working_directory:
			if filename.startswith(self.working_directory):
				
				cut = len(self.working_directory)
				if self.working_directory[-1] != "/":
					cut += 1
					
				filename = filename[cut:]
		
		
		includegraphics = Command("includegraphics")
		if len(options):
			includegraphics.arguments.append(OptionalArgument(options))
		includegraphics.arguments.append(MandatoryArgument(filename))
		
		
		root = includegraphics
		
		
		# scale with h != v
		if (self.h_scale != 1.0 or self.v_scale != 1.0) and self.h_scale != self.v_scale:
			scalebox = Command("scalebox")
			scalebox.arguments.append(MandatoryArgument(round(self.h_scale, 2)))
			scalebox.arguments.append(MandatoryArgument(round(self.v_scale, 2)))
			scalebox.arguments.append(MandatoryArgument(root))
			
			root = scalebox
		
		# reflect
		if self.reflect:
			reflectbox = Command("reflectbox")
			reflectbox.arguments.append(MandatoryArgument(root))
			
			root = reflectbox
		
		center = Environment("center")
		center.append(root)
		
		if self.float:
			figure = Environment("figure")
			
			# placement
			p = []
				
			if self.placement_b:
				p.append("b")
			if self.placement_h:
				p.append("h")
			if self.placement_p:
				p.append("p")
			if self.placement_t:
				p.append("t")
			
			if len(p):
				figure.arguments.append(OptionalArgument("".join(p)))
			
			figure.append(center)
			
			if self.caption:
				caption = Command("caption")
				caption.arguments.append(MandatoryArgument(self.caption))
				figure.append(caption)
			
			if self.label:
				label = Command("label")
				label.arguments.append(MandatoryArgument(self.label))
				figure.append(label)
		
			return ConditionalSource(str(figure), ["graphicx"])
		else:
			packages = ["graphicx"]
			
			if self.caption:
				captionof = Command("captionof")
				captionof.arguments.append(MandatoryArgument("figure"))
				captionof.arguments.append(MandatoryArgument(self.caption))
				center.append(captionof)
				packages.append("caption")
			
			if self.label:
				label = Command("label")
				label.arguments.append(MandatoryArgument(self.label))
				center.append(label)
			
			return ConditionalSource(str(center), packages)
		
	
#	def _get_source(self):
#		if not self.filename:
#			return
#		
#		options = []
#		
#		# scale with h == v
#		if (self.h_scale != 1.0 or self.v_scale != 1.0) and self.h_scale == self.v_scale:
#			options.append("scale=%s" % round(self.h_scale, 2))
#		
#		# resize
#		if self.resize_width:
#			options.append("width=%s%s" % (round(self.resize_width, 2), self.resize_unit))
#		
#		if self.resize_width:
#			options.append("height=%s%s" % (round(self.resize_height, 2), self.resize_unit))
#			
#		# rotate
#		if self.rotate != 0.0:
#			options.append("angle=%s" % round(self.rotate, 2))
#		
#		# try relativize filename to work directory
#		filename = self.filename
#		
#		if self.working_directory:
#			if filename.startswith(self.working_directory):
#				
#				cut = len(self.working_directory)
#				if self.working_directory[-1] != "/":
#					cut += 1
#					
#				filename = filename[cut:]
#		
#		if len(options):
#			source = "\\includegraphics[%s]{%s}" % (",".join(options), filename)
#		else:
#			source = "\\includegraphics{%s}" % filename
#		
#		# scale with h != v
#		if (self.h_scale != 1.0 or self.v_scale != 1.0) and self.h_scale != self.v_scale:
#			source = "\\scalebox{%s}{%s}{%s}" % (round(self.h_scale, 2), round(self.v_scale, 2), source)
#		
#		# reflect
#		if self.reflect:
#			source = "\\reflectbox{%s}" % source
#		
#		# center
#		if self.center:
#			source = "\\centering\n" + source
#		
#		# floating object
#		if self.float:
#			# caption
#			if self.caption and len(self.caption):
#				source += "\n\\caption{%s}" % self.caption
#			
#			# label
#			if self.label and len(self.label):
#				source += "\n\\label{%s}" % self.label
#		
#			# placement
#			pl = ""
#			
#			if self.placement_b or self.placement_h or self.placement_p or self.placement_t:
#				pl = "["
#				
#				if self.placement_b:
#					pl += "b"
#				if self.placement_h:
#					pl += "h"
#				if self.placement_p:
#					pl += "p"
#				if self.placement_t:
#					pl += "t"
#				
#				pl += "]"
#			
#			# figure
#			source = "\\begin{figure}%s\n%s\n\\end{figure}" % (pl, source)
#		
#		return source
