# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import ScrolledWindow, ListStore, TreeView, CellRendererText, POLICY_AUTOMATIC, SHADOW_NONE, \
	Toolbar, ToolButton, STOCK_ADD, VBox, Tooltips, STOCK_REMOVE, STOCK_EDIT, RESPONSE_OK, \
	DEST_DEFAULT_ALL, TOOLBAR_BOTH_HORIZ
from gtk.gdk import ACTION_COPY
from gobject import SIGNAL_RUN_LAST, TYPE_NONE, TYPE_INT, TYPE_STRING, TYPE_PYOBJECT

class SnippetView(VBox):
	"""
	"""
	
	__gsignals__ = {
		"snippet-activated" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_INT])),
		"new-clicked" : (SIGNAL_RUN_LAST, TYPE_NONE, ([])),
		"remove-clicked" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_INT])),
		"edit-clicked" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_INT])),
		"drag-text-received" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_STRING]))
	}
	
	def __init__(self):
		VBox.__init__(self)
		
		# toolbar
		self.tooltips = Tooltips()
		
		toolbar = Toolbar()
		toolbar.set_style(TOOLBAR_BOTH_HORIZ)
		
		button_add = ToolButton(STOCK_ADD)
		button_add.set_tooltip(self.tooltips, _("Add a new snippet..."))
		button_add.connect("clicked", lambda x: self.emit("new-clicked"))
		toolbar.insert(button_add, -1)
		
		button_remove = ToolButton(STOCK_REMOVE)
		button_remove.set_tooltip(self.tooltips, _("Remove selected snippet"))
		button_remove.connect("clicked", self.__remove_clicked)
		toolbar.insert(button_remove, -1)
		
		button_edit = ToolButton(STOCK_EDIT)
		button_edit.set_tooltip(self.tooltips, _("Edit selected snippet..."))
		button_edit.connect("clicked", self.__edit_clicked)
		toolbar.insert(button_edit, -1)
		
		self.pack_start(toolbar, False)
		
		# list
		self.__store = ListStore(str)
		
		self.view_snippets = TreeView(self.__store)
		self.view_snippets.set_headers_visible(False)
		self.view_snippets.insert_column_with_attributes(-1, "", CellRendererText(), markup=0)
		self.view_snippets.connect("row-activated", self.__snippet_activated)
		
		self.view_snippets.drag_dest_set(DEST_DEFAULT_ALL, [("STRING", 0, 0)], ACTION_COPY)
		self.view_snippets.connect("drag_data_received", self.__drag_data_received)
		
		scrolled = ScrolledWindow()
		scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
		scrolled.set_shadow_type(SHADOW_NONE)
		
		scrolled.add(self.view_snippets)
		
		self.pack_start(scrolled, True)
		
		self.show_all()
		
	def set_snippets(self, snippets):
		self.__store.clear()
		for snippet in snippets:
			self.__store.append([snippet.get_markup()])
	
	def __drag_data_received(self, widget, drag_context, x, y, selection_data, info, timestamp):
		text = selection_data.get_text()
		self.emit("drag-text-received", text)
	
	def __snippet_activated(self, treeview, path, column):
		self.emit("snippet-activated", path[0])
	
	def __get_selected_row(self):
		store, it = self.view_snippets.get_selection().get_selected()
		if not it:
			raise IndexError
		path = store.get_path(it)
		return path[0]
	
	def __remove_clicked(self, toolbutton):
		try:
			row = self.__get_selected_row()
			self.emit("remove-clicked", row)
		except IndexError:
			return
	
	def __edit_clicked(self, toolbutton):
		try:
			row = self.__get_selected_row()
			self.emit("edit-clicked", row)
		except IndexError:
			return
