# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import ScrolledWindow, POLICY_AUTOMATIC, SHADOW_NONE, VBox, Expander, \
		Label, ListStore, IconView, SELECTION_SINGLE, SizeGroup, SIZE_GROUP_HORIZONTAL, \
		Button, RELIEF_NONE, Image, Widget, POLICY_NEVER, Label, gtk_version
from gtk.gdk import Pixbuf, pixbuf_new_from_file
from gobject import GError, SIGNAL_RUN_LAST, TYPE_NONE, TYPE_INT, TYPE_STRING, TYPE_PYOBJECT, \
	TYPE_BOOLEAN
	
from LaTeXPlugin.Logger import debug, error 

	
class SymbolView(ScrolledWindow):
	"""
	New view for the symbols
	
	Display the groups of symbols in expanders and take 
	them from a symbols.xml
	"""
	
	__gsignals__ = {
		# def callback(symbol_view, source)
		"symbol-selected" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_PYOBJECT])),
		# def callback(group_label, expanded)
		"group-expanded" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_STRING, TYPE_BOOLEAN]))
	}
	
	def __init__(self):
		ScrolledWindow.__init__(self)
		self.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.set_shadow_type(SHADOW_NONE)
		
		self.__vbox = VBox()
		
		self.add_with_viewport(self.__vbox)
		
		self.show_all()
	
	def add_group(self, group):
		model = ListStore(Pixbuf, str)
		
		for symbol in group.symbols:
			try:
				model.append([pixbuf_new_from_file(symbol.icon), symbol.source])
			except GError, s:
				print s
		
		view = IconView(model)
		view.set_pixbuf_column(0)
		view.set_selection_mode(SELECTION_SINGLE)
		view.connect("selection-changed", self.__symbol_selected)
		view.set_item_width(-1)			# use largest item width
		view.set_spacing(0)
		view.set_column_spacing(0)
		view.set_row_spacing(0)
		view.set_columns(-1)			# show as much as possible
		view.set_text_column(-1)
		view.show()
		
		if gtk_version >= (2, 10, 14):
			expander = Expander(group.label)
			expander.add(view)
			expander.show_all()
			expander.set_expanded(group.visible)
			expander.connect("notify::expanded", self.__group_expanded, group.label)
			
			# FIXME: Works with gtk+ 2.10.14 and pygtk 2.10.6. With gtk 2.10.11 and 
			# pygtk 2.10.4 the expanders don't acquire any space on this way.
			# It only works with self.__vbox.pack_start(expander, True) but that doesn't make
			# sense...
			self.__vbox.pack_start(expander, False)
		else:
			#error(self, "add_group", "gtk+ >= 2.10.14 is needed to avoid some layout problems here, using labels instead of expanders.")
			
			label = Label(group.label)
			label.set_alignment(0, .5)
			label.show()
			
			self.__vbox.pack_start(label, False)
			self.__vbox.pack_start(view, True)
			
			self.__vbox.set_spacing(3)
	
	def __group_expanded(self, expander, param_spec, group_label):
		self.emit("group-expanded", group_label, expander.get_expanded())
	
	def __symbol_selected(self, iconview):
		try: 
			path = iconview.get_selected_items()[0]
			source = iconview.get_model()[path][1]
			#
			# TODO: pass a source object
			#
			self.emit("symbol-selected", source)
			
			iconview.unselect_all()
		except IndexError:
			pass				# must be caught after unselect_all()
		