# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Dialog, DIALOG_MODAL, DIALOG_DESTROY_WITH_PARENT, STOCK_CANCEL, RESPONSE_CANCEL, \
		STOCK_OK, RESPONSE_OK, Label, RadioButton, FileChooserDialog, FileChooserButton, \
		FILE_CHOOSER_ACTION_OPEN, STOCK_OPEN, RESPONSE_ACCEPT, SpinButton, Adjustment, ListStore, \
		ComboBox, CellRendererPixbuf, CellRendererText, combo_box_entry_new_text, Table, Notebook, \
		ScrolledWindow, POLICY_AUTOMATIC, SHADOW_NONE, CheckButton, Entry, HBox, ComboBoxEntry
from gtk.gdk import Pixbuf, pixbuf_new_from_file

from LaTeXPlugin.Installation import Installation
from LaTeXPlugin.Logger import error, debug
from LaTeXPlugin.PreviewArea import PreviewArea
from LaTeXPlugin.common import ConditionalSource, TEXT, MATH

from TableSourceBuilder import TableSourceBuilder, GridBuilder

class TableDialog(Dialog):
	def __init__(self, parent):
		Dialog.__init__(self, "Insert Tabular Object", parent, DIALOG_MODAL | DIALOG_DESTROY_WITH_PARENT, \
					(STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		
		self.set_border_width(5)
		self.set_has_separator(False)
		#self.set_resizable(False)
		#self.vbox.set_spacing(10)
		
		self.__source_builder = TableSourceBuilder()
		self.__grid_builder = GridBuilder()
		
		# content
		lbl_content = Label("<b>Content</b>")
		lbl_content.set_use_markup(True)
		lbl_content.set_alignment(0, .5)
		
		self.rad_placeholders = RadioButton(None, "Insert _placeholders", True)
		self.rad_placeholders.connect("toggled", self.__content_toggled)
		
		self.rad_import = RadioButton(self.rad_placeholders, "Import from _file:", True)
		
		self.dlg_file = FileChooserDialog(action=FILE_CHOOSER_ACTION_OPEN, buttons=(STOCK_OPEN, RESPONSE_ACCEPT, STOCK_CANCEL, RESPONSE_CANCEL))
		self.dlg_file.set_local_only(True)
		self.dlg_file.connect("response", self.__file_response)
		
		self.btn_file = FileChooserButton(self.dlg_file)
		self.btn_file.set_sensitive(False)
		
		# layout
		lbl_layout = Label("<b>Layout</b>")
		lbl_layout.set_use_markup(True)
		lbl_layout.set_alignment(0, .5)
		
		self.spn_rows = SpinButton(Adjustment(1, 1, 1000, 1))
		
		lbl_rows = Label("_Rows:")
		lbl_rows.set_use_underline(True)
		lbl_rows.set_alignment(1, .5)
		lbl_rows.set_mnemonic_widget(self.spn_rows)
		
		self.spn_columns = SpinButton(Adjustment(1, 1, 1000, 1))
		self.spn_columns.connect("value-changed", self.__columns_changed)
		
		lbl_columns = Label("_Columns:")
		lbl_columns.set_use_underline(True)
		lbl_columns.set_alignment(0, .5)
		lbl_columns.set_mnemonic_widget(self.spn_columns)
		
		box_layout = HBox()
		box_layout.set_spacing(10)
		box_layout.pack_start(lbl_rows, False)
		box_layout.pack_start(self.spn_rows, False)
		box_layout.pack_start(lbl_columns, False)
		box_layout.pack_start(self.spn_columns, False)
		
		# context
		lbl_context = Label("<b>Context</b>")
		lbl_context.set_use_markup(True)
		lbl_context.set_alignment(0, .5)
		
		self.rad_text = RadioButton(None, "_Text (table)", True)
		self.rad_text.connect("toggled", self.__context_toggled)
		
		self.rad_math = RadioButton(self.rad_text, "_Math (matrix)", True)
		
		
		# column style
		
		self.__sto_column_style = ListStore(str)
		self.__sto_column_style.append(["l"])
		
		self.__cmb_column_style = ComboBoxEntry(self.__sto_column_style, 0)
		
#		rnd_text = CellRendererText()
#		rnd_text.set_property("family", "Monospace")
#		
#		self.__cmb_column_style.pack_start(rnd_text, True)
#  		self.__cmb_column_style.add_attribute(rnd_text, 'text', 0)
		
		self.__cmb_column_style.set_active(0)
		
		
		self.lbl_col_style = Label("Column _style:")
		self.lbl_col_style.set_use_underline(True)
		self.lbl_col_style.set_alignment(0, .5)
		self.lbl_col_style.set_mnemonic_widget(self.__cmb_column_style)
		
		# delimiters
		self.__store_delims = ListStore(Pixbuf, str, str)	# icon, label, environment
		
		self.__store_delims.append([None, "None", "matrix"])
		
		delimiters = [("parantheses", "Parantheses", "pmatrix"), 
					("brackets", "Brackets", "bmatrix"), 
					("braces", "Braces", "Bmatrix"), 
					("vbars", "Vertical Bars", "vmatrix"), 
					("dvbars", "Double Vertical Bars", "Vmatrix")]
		
		for d in delimiters:
			pixbuf = pixbuf_new_from_file("%s/pixmaps/%s.png" % (Installation().path, d[0]))
			self.__store_delims.append([pixbuf, d[1], d[2]])
		
		self.cmb_delims = ComboBox(self.__store_delims)
		
		cellPixbuf = CellRendererPixbuf()
  		self.cmb_delims.pack_start(cellPixbuf, False)
  		self.cmb_delims.add_attribute(cellPixbuf, 'pixbuf', 0)
  		
		cellText = CellRendererText()
  		self.cmb_delims.pack_start(cellText, False)
  		self.cmb_delims.add_attribute(cellText, 'text', 1)
  		
  		self.cmb_delims.set_active(0)
  		self.cmb_delims.set_sensitive(False)
  		
  		self.lbl_delims = Label("De_limiters:")
  		self.lbl_delims.set_use_underline(True)
  		self.lbl_delims.set_alignment(0, .5)
  		self.lbl_delims.set_mnemonic_widget(self.cmb_delims)
  		self.lbl_delims.set_sensitive(False)
	  	
	  	# floating
		lbl_float = Label("<b>Floating</b>")
		lbl_float.set_alignment(0, .5)
		lbl_float.set_use_markup(True)
		
		self.chk_float = CheckButton("_Float", True)
		self.chk_float.set_active(True)
		#self.chk_float.connect("toggled", self.__float_toggled)
		
		self.ent_caption = Entry()
		self.ent_caption.set_activates_default(True)
		
		self.lbl_caption = Label("_Caption:")
		self.lbl_caption.set_alignment(0, .5)
		self.lbl_caption.set_use_underline(True)
		self.lbl_caption.set_mnemonic_widget(self.ent_caption)
		
		self.ent_label = Entry()
		self.ent_label.set_activates_default(True)
		
		self.lbl_label = Label("_Label:")
		self.lbl_label.set_alignment(0, .5)
		self.lbl_label.set_use_underline(True)
		self.lbl_label.set_mnemonic_widget(self.ent_label)
	  	
	  	tbl = Table()
	  	tbl.attach(lbl_content, 0, 5, 0, 1)
	  	tbl.attach(self.rad_placeholders, 1, 5, 1, 2)
	  	tbl.attach(self.rad_import, 1, 2, 2, 3)
	  	tbl.attach(self.btn_file, 2, 5, 2, 3, yoptions=0)
	  	tbl.attach(lbl_layout, 0, 5, 3, 4)
	  	#tbl.attach(lbl_rows, 1, 2, 4, 5)
	  	#tbl.attach(self.spn_rows, 2, 3, 4, 5)
	  	#tbl.attach(lbl_columns, 3, 4, 4, 5)
	  	#tbl.attach(self.spn_columns, 4, 5, 4, 5)
	  	tbl.attach(box_layout, 1, 5, 4, 5)
	  	tbl.attach(lbl_context, 0, 5, 5, 6)
	  	tbl.attach(self.rad_text, 1, 2, 6, 7)
	  	tbl.attach(self.lbl_col_style, 1, 2, 7, 8)
	  	tbl.attach(self.__cmb_column_style, 2, 5, 7, 8, yoptions=0)
	  	tbl.attach(self.rad_math, 1, 2, 8, 9)
	  	tbl.attach(self.lbl_delims, 1, 2, 9, 10)
	  	tbl.attach(self.cmb_delims, 2, 5, 9, 10, yoptions=0)
	  	#tbl.attach(lbl_float, 0, 5, 10, 11)
	  	tbl.attach(self.chk_float, 0, 5, 10, 11)
	  	tbl.attach(self.lbl_caption, 0, 2, 11, 12)
	  	tbl.attach(self.ent_caption, 2, 5, 11, 12)
	  	tbl.attach(self.lbl_label, 0, 2, 12, 13)
	  	tbl.attach(self.ent_label, 2, 5, 12, 13)
	  	
	  	tbl.set_border_width(10)
		tbl.set_row_spacings(5)
		tbl.set_col_spacings(5)
		tbl.set_col_spacing(0, 15)
		tbl.set_row_spacing(2, 10)
		tbl.set_row_spacing(4, 10)
		tbl.set_row_spacing(9, 10)
	  	
	  	# preview
	  	self.__preview_area = PreviewArea()
	  	
	  	scr_preview = ScrolledWindow()
		scr_preview.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		scr_preview.set_shadow_type(SHADOW_NONE)
		scr_preview.add_with_viewport(self.__preview_area)
	  	
	  	# notebook
	  	ntb = Notebook()
		ntb.set_border_width(5)
		ntb.append_page(tbl, Label("Properties"))
		ntb.append_page(scr_preview, Label("Preview"))
		ntb.connect("switch-page", self.__page_switched)
	  	
	  	ntb.show_all()
	  	
	  	self.vbox.add(ntb)
	
	def __columns_changed(self, spin_button):
		""" suggest layouts that fit the number of columns """
		cols = spin_button.get_value_as_int()
		
#		self.cmb_col_style.get_model().clear()
#		
#		for style in ["l" * cols, "|".join("l" * cols), "@{\\ \\vline\\ }".join("l" * cols)]:
#			self.cmb_col_style.append_text(style)
		
		self.__sto_column_style.clear()
		
		for style in ["l" * cols, "|".join("l" * cols), "@{\\ \\vline\\ }".join("l" * cols)]:
			self.__sto_column_style.append([style])
		
		self.__cmb_column_style.set_active(0)
	
#	def __float_toggled(self, toggle_button):
#		v = toggle_button.get_active()
#		
#		self.lbl_caption.set_sensitive(v)
#		self.ent_caption.set_sensitive(v)
#		self.lbl_label.set_sensitive(v)
#		self.ent_label.set_sensitive(v)
	
	def __content_toggled(self, radio_button):
		if self.rad_placeholders.get_active():
			self.__unfreeze_layout()
			self.btn_file.set_sensitive(False)
		else:
			self.btn_file.set_sensitive(True)
			
	def __context_toggled(self, radio_button):
		self.lbl_col_style.set_sensitive(self.rad_text.get_active())
		self.cmb_col_style.set_sensitive(self.rad_text.get_active())
		
		self.lbl_delims.set_sensitive(self.rad_math.get_active())
		self.cmb_delims.set_sensitive(self.rad_math.get_active())
	
	def __page_switched(self, notebook, page, page_num):
		if page_num == 1:
			# generate preview
			
			source = self.source
			
			if type(source) is str:
				debug(self, "__page_switched", "<<<%s>>>" % source)
				source_s = source
			elif type(source) is ConditionalSource:
				debug(self, "__page_switched", "<<<%s>>>" % source.source)
				source_s = source.source
				self.__preview_area.set_packages(source.packages)
			
			if self.__source_builder.context_math:
				source_s = "$%s$" % source_s
				
			self.__preview_area.set_source(source_s)
			self.__preview_area.render()
	
	def __file_response(self, dialog, response):
		if response == RESPONSE_ACCEPT:
			# try to create content
			try:
				self.__grid_builder.load_from_file(dialog.get_filename())
				
				self.__freeze_layout(self.__grid_builder.n_rows, self.__grid_builder.n_columns)
				
			except Exception, s:
				# TODO: popup message
				error(self, "__file_response", s)
				return
	
	def __freeze_layout(self, n_rows, n_columns):
		self.spn_rows.set_value(n_rows)
		self.spn_columns.set_value(n_columns)
		
		self.spn_rows.set_sensitive(False)
		self.spn_columns.set_sensitive(False)
	
	def __unfreeze_layout(self):
		self.spn_rows.set_sensitive(True)
		self.spn_columns.set_sensitive(True)
	
	def __reset(self):
		self.dlg_file.unselect_all()
		self.spn_rows.set_sensitive(True)
		self.spn_columns.set_sensitive(True)
		self.rad_placeholders.set_active(True)
		self.ent_caption.set_text("")
		self.ent_label.set_text("")
	
	@property
	def source(self):
		# TODO: improve and simplify this
		
		# grid/layout
		if self.rad_import.get_active():
			self.__grid_builder.load_from_file(self.dlg_file.get_filename())
			
			self.__source_builder.n_rows = self.__grid_builder.n_rows
			self.__source_builder.n_columns = self.__grid_builder.n_columns
		else:
			self.__source_builder.n_rows = self.spn_rows.get_value_as_int()
			self.__source_builder.n_columns = self.spn_columns.get_value_as_int()
			
			self.__grid_builder.create_placeholders(self.__source_builder.n_rows, \
												self.__source_builder.n_columns)
		
		self.__source_builder.grid = self.__grid_builder.grid
		
		# context
		self.__source_builder.context_text = self.rad_text.get_active()
		self.__source_builder.column_style = self.__cmb_column_style.get_active_text()
		
		self.__source_builder.context_math = self.rad_math.get_active()
		i = self.cmb_delims.get_active()
		self.__source_builder.math_environment = self.__store_delims[i][2]
		
		# float
		self.__source_builder.float = self.chk_float.get_active()
		
		self.__source_builder.label = self.ent_label.get_text()
		self.__source_builder.caption = self.ent_caption.get_text()
		
		return self.__source_builder.get_source()
	
	def run(self, environment):
		self.__reset()
		
		self.rad_text.set_active(environment == TEXT)
		self.rad_math.set_active(environment == MATH)
		
		return Dialog.run(self)
	